@@ -457,6 +457,9 @@ class WarningsFixer(object):
     An object "fixing" warnings of a given category caught during
     certain phases.  The warnings can have their filename and lineno fixed,
     and they are deduplicated as well.
+
+    When used as a context manager, any warnings caught by `.catch_warnings()`
+    will be flushed at the exit of the context manager.
     """
 
     def __init__(self, category):
@@ -502,6 +505,12 @@ class WarningsFixer(object):
                 warnings.warn_explicit(msg, category, filename, lineno)
         self._warnings.clear()
 
+    def __enter__(self):
+        return
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.flush()
+
 
 class NumbaError(Exception):
 
